@@ -1,18 +1,34 @@
 #include "livetvchain.h"
 #include "mythcontext.h"
-#include "mythdbcon.h"
+#include "mythdb.h"
+#include "mythlogging.h"
 #include "programinfo.h"
 #include "mythsocket.h"
+#include "cardutil.h"
 
 #define LOC QString("LiveTVChain(%1): ").arg(m_id)
 
+static inline void clear(LiveTVChainEntry &entry)
+{
+    entry.chanid = 0;
+    entry.starttime.setTime_t(0);
+    entry.endtime = QDateTime();
+    entry.discontinuity = true;
+    entry.hostprefix = QString();
+    entry.cardtype = QString();
+    entry.channum = QString();
+    entry.inputname = QString();
+}
+
 /** \class LiveTVChain
  *  \brief Keeps track of recordings in a current LiveTV instance
  */
-LiveTVChain::LiveTVChain()
-           : m_id(""), m_maxpos(0), m_lock(true), m_curpos(0), m_cur_chanid(""),
-             m_switchid(-1), m_jumppos(0)
+LiveTVChain::LiveTVChain() :
+    m_id(""), m_maxpos(0), m_lock(QMutex::Recursive),
+    m_curpos(0), m_cur_chanid(0),
+    m_switchid(-1), m_jumppos(0)
 {
+    clear(m_switchentry);
 }
 
 LiveTVChain::~LiveTVChain()
@@ -21,7 +37,7 @@ LiveTVChain::~LiveTVChain()
 
 QString LiveTVChain::InitializeNewChain(const QString &seed)
 {
-    QDateTime curdt = QDateTime::currentDateTime();
+    QDateTime curdt = MythDate::current();
     m_id = QString("live-%1-%2").arg(seed).arg(curdt.toString(Qt::ISODate));
     return m_id;
 }
@@ -47,13 +63,13 @@ void LiveTVChain::AppendNewProgram(ProgramInfo *pginfo, QString channum,
 {
     QMutexLocker lock(&m_lock);
 
-    QTime tmptime = pginfo->recstartts.time();
+    QTime tmptime = pginfo->GetRecordingStartTime().time();
 
     LiveTVChainEntry newent;
-    newent.chanid = pginfo->chanid;
-    newent.starttime = pginfo->recstartts;
-    newent.starttime.setTime(QTime(tmptime.hour(), tmptime.minute(), 
-                                   tmptime.second()));
+    newent.chanid = pginfo->GetChanID();
+    newent.starttime = QDateTime(
+        pginfo->GetRecordingStartTime().date(),
+        QTime(tmptime.hour(), tmptime.minute(), tmptime.second()), Qt::UTC);
     newent.discontinuity = discont;
     newent.hostprefix = m_hostprefix;
     newent.cardtype = m_cardtype;
@@ -69,25 +85,25 @@ void LiveTVChain::AppendNewProgram(ProgramInfo *pginfo, QString channum,
                   "VALUES(:CHANID, :START, :END, :CHAINID, :CHAINPOS, "
                   " :DISCONT, :WATCHING, :PREFIX, :CARDTYPE, :CHANNAME, "
                   " :INPUT );");
-    query.bindValue(":CHANID", pginfo->chanid);
-    query.bindValue(":START", pginfo->recstartts);
-    query.bindValue(":END", pginfo->recendts);
+    query.bindValue(":CHANID", pginfo->GetChanID());
+    query.bindValue(":START", pginfo->GetRecordingStartTime());
+    query.bindValue(":END", pginfo->GetRecordingEndTime());
     query.bindValue(":CHAINID", m_id);
     query.bindValue(":CHAINPOS", m_maxpos);
     query.bindValue(":DISCONT", discont);
     query.bindValue(":WATCHING", 0);
     query.bindValue(":PREFIX", m_hostprefix);
     query.bindValue(":CARDTYPE", m_cardtype);
-    query.bindValue(":CHANNAME", channum.utf8());
-    query.bindValue(":INPUT", inputname.utf8());
+    query.bindValue(":CHANNAME", channum);
+    query.bindValue(":INPUT", inputname);
 
     if (!query.exec() || !query.isActive())
-        MythContext::DBError("Chain: AppendNewProgram", query);
+        MythDB::DBError("Chain: AppendNewProgram", query);
     else
-        VERBOSE(VB_RECORD, QString("Chain: Appended@%3 '%1_%2'")
-                .arg(newent.chanid)
-                .arg(newent.starttime.toString("yyyyMMddhhmmss"))
-                .arg(m_maxpos));
+        LOG(VB_RECORD, LOG_INFO, QString("Chain: Appended@%3 '%1_%2'")
+            .arg(newent.chanid)
+            .arg(MythDate::toString(newent.starttime, MythDate::kFilename))
+            .arg(m_maxpos));
 
     m_maxpos++;
     BroadcastUpdate();
@@ -100,25 +116,26 @@ void LiveTVChain::FinishedRecording(ProgramInfo *pginfo)
     MSqlQuery query(MSqlQuery::InitCon());
     query.prepare("UPDATE tvchain SET endtime = :END "
                   "WHERE chanid = :CHANID AND starttime = :START ;");
-    query.bindValue(":END", pginfo->recendts);
-    query.bindValue(":CHANID", pginfo->chanid);
-    query.bindValue(":START", pginfo->recstartts);
+    query.bindValue(":END", pginfo->GetRecordingEndTime());
+    query.bindValue(":CHANID", pginfo->GetChanID());
+    query.bindValue(":START", pginfo->GetRecordingStartTime());
 
     if (!query.exec() || !query.isActive())
-        MythContext::DBError("Chain: FinishedRecording", query);
+        MythDB::DBError("Chain: FinishedRecording", query);
     else
-        VERBOSE(VB_RECORD, QString("Chain: Updated endtime for '%1_%2' to %3")
-                .arg(pginfo->chanid)
-                .arg(pginfo->recstartts.toString("yyyyMMddhhmmss"))
-                .arg(pginfo->recendts.toString("yyyyMMddhhmmss")));
+        LOG(VB_RECORD, LOG_INFO,
+            QString("Chain: Updated endtime for '%1_%2' to %3")
+                .arg(pginfo->GetChanID())
+                .arg(pginfo->GetRecordingStartTime(MythDate::kFilename))
+                .arg(pginfo->GetRecordingEndTime(MythDate::kFilename)));
 
-    QValueList<LiveTVChainEntry>::iterator it;
+    QList<LiveTVChainEntry>::iterator it;
     for (it = m_chain.begin(); it != m_chain.end(); ++it)
     {
-        if ((*it).chanid == pginfo->chanid &&
-            (*it).starttime == pginfo->recstartts)
+        if ((*it).chanid    == pginfo->GetChanID() &&
+            (*it).starttime == pginfo->GetRecordingStartTime())
         {
-            (*it).endtime = pginfo->recendts;
+            (*it).endtime = pginfo->GetRecordingEndTime();
         }
     }
     BroadcastUpdate();
@@ -128,11 +145,11 @@ void LiveTVChain::DeleteProgram(ProgramInfo *pginfo)
 {
     QMutexLocker lock(&m_lock);
 
-    QValueList<LiveTVChainEntry>::iterator it, del;
+    QList<LiveTVChainEntry>::iterator it, del;
     for (it = m_chain.begin(); it != m_chain.end(); ++it)
     {
-        if ((*it).chanid == pginfo->chanid &&
-            (*it).starttime == pginfo->recstartts)
+        if ((*it).chanid    == pginfo->GetChanID() &&
+            (*it).starttime == pginfo->GetRecordingStartTime())
         {
             del = it;
             ++it;
@@ -148,7 +165,9 @@ void LiveTVChain::DeleteProgram(ProgramInfo *pginfo)
                 query.bindValue(":START", (*it).starttime);
                 query.bindValue(":CHAINID", m_id);
                 query.bindValue(":DISCONT", true);
-                query.exec();
+                if (!query.exec())
+                    MythDB::DBError("LiveTVChain::DeleteProgram -- "
+                                    "discontinuity", query);
             }
 
             query.prepare("DELETE FROM tvchain WHERE chanid = :CHANID "
@@ -156,9 +175,10 @@ void LiveTVChain::DeleteProgram(ProgramInfo *pginfo)
             query.bindValue(":CHANID", (*del).chanid);
             query.bindValue(":START", (*del).starttime);
             query.bindValue(":CHAINID", m_id);
-            query.exec();
+            if (!query.exec())
+                MythDB::DBError("LiveTVChain::DeleteProgram -- delete", query);
 
-            m_chain.remove(del);
+            m_chain.erase(del);
 
             BroadcastUpdate();
             break;
@@ -170,7 +190,7 @@ void LiveTVChain::BroadcastUpdate(void)
 {
     QString message = QString("LIVETV_CHAIN UPDATE %1").arg(m_id);
     MythEvent me(message);
-    gContext->dispatch(me);
+    gCoreContext->dispatch(me);
 }
 
 void LiveTVChain::DestroyChain(void)
@@ -183,14 +203,15 @@ void LiveTVChain::DestroyChain(void)
     query.prepare("DELETE FROM tvchain WHERE chainid = :CHAINID ;");
     query.bindValue(":CHAINID", m_id);
 
-    query.exec();
+    if (!query.exec())
+        MythDB::DBError("LiveTVChain::DestroyChain", query);
 }
 
 void LiveTVChain::ReloadAll(void)
 {
     QMutexLocker lock(&m_lock);
 
-    uint prev_size = m_chain.size();
+    int prev_size = m_chain.size();
     m_chain.clear();
 
     MSqlQuery query(MSqlQuery::InitCon());
@@ -206,14 +227,14 @@ void LiveTVChain::ReloadAll(void)
         {
 
             LiveTVChainEntry entry;
-            entry.chanid = query.value(0).toString();
-            entry.starttime = query.value(1).toDateTime();
-            entry.endtime = query.value(2).toDateTime();
+            entry.chanid = query.value(0).toUInt();
+            entry.starttime = MythDate::as_utc(query.value(1).toDateTime());
+            entry.endtime = MythDate::as_utc(query.value(2).toDateTime());
             entry.discontinuity = query.value(3).toInt();
             entry.hostprefix = query.value(5).toString();
             entry.cardtype = query.value(6).toString();
-            entry.channum = QString::fromUtf8(query.value(7).toString());
-            entry.inputname = QString::fromUtf8(query.value(8).toString());
+            entry.channum = query.value(7).toString();
+            entry.inputname = query.value(8).toString();
 
             m_maxpos = query.value(4).toInt() + 1;
 
@@ -230,8 +251,8 @@ void LiveTVChain::ReloadAll(void)
 
     if (prev_size!=m_chain.size())
     {
-        VERBOSE(VB_PLAYBACK, LOC +
-                "ReloadAll(): Added new recording");
+        LOG(VB_PLAYBACK, LOG_INFO, LOC + "ReloadAll(): Added new recording");
+        LOG(VB_PLAYBACK, LOG_INFO, LOC + toString());
     }
 }
 
@@ -240,33 +261,39 @@ void LiveTVChain::GetEntryAt(int at, LiveTVChainEntry &entry) const
     QMutexLocker lock(&m_lock);
 
     int size = m_chain.count();
-    int new_at = (at < 0 || at >= size) ? size - 1 : at;
+    int new_at = (size && (at < 0 || at >= size)) ? size - 1 : at;
 
-    if (new_at >= 0 && new_at <= size)
+    if (size && new_at >= 0 && new_at < size)
         entry = m_chain[new_at];
     else
     {
-        VERBOSE(VB_IMPORTANT, QString("GetEntryAt(%1) failed.").arg(at));
-        entry.chanid = "0";
-        entry.starttime.setTime_t(0);
+        LOG(VB_GENERAL, LOG_ERR, QString("GetEntryAt(%1) failed.").arg(at));
+        if (at == -1)
+            LOG(VB_GENERAL, LOG_ERR, "It appears that your backend may "
+                "be misconfigured.  Check your backend logs to determine "
+                "whether your capture cards, lineups, channels, or storage "
+                "configuration are reporting errors.  This issue is commonly "
+                "caused by failing to complete all setup steps properly.  You "
+                "may wish to review the documentation for mythtv-setup.");
+        clear(entry);
     }
 }
 
 ProgramInfo *LiveTVChain::EntryToProgram(const LiveTVChainEntry &entry)
 {
-    ProgramInfo *pginfo;
-    pginfo = ProgramInfo::GetProgramFromRecorded(entry.chanid,
-                                                 entry.starttime);
-    if (pginfo)
-        pginfo->pathname = entry.hostprefix + pginfo->pathname;
-    else
+    ProgramInfo *pginfo = new ProgramInfo(entry.chanid, entry.starttime);
+
+    if (pginfo->GetChanID())
     {
-        VERBOSE(VB_IMPORTANT,
-                QString("EntryToProgram(%1@%2) failed to get pginfo")
-                .arg(entry.chanid).arg(entry.starttime.toString()));
+        pginfo->SetPathname(entry.hostprefix + pginfo->GetBasename());
+        return pginfo;
     }
 
-    return pginfo;
+    LOG(VB_GENERAL, LOG_ERR,
+        QString("EntryToProgram(%1@%2) failed to get pginfo")
+        .arg(entry.chanid).arg(entry.starttime.toString(Qt::ISODate)));
+    delete pginfo;
+    return NULL;
 }
 
 /** \fn LiveTVChain::GetProgramAt(int) const
@@ -284,16 +311,15 @@ ProgramInfo *LiveTVChain::GetProgramAt(int at) const
     return EntryToProgram(entry);
 }
 
-/** \fn LiveTVChain::ProgramIsAt(const QString&, const QDateTime&) const
+/** \fn LiveTVChain::ProgramIsAt(uint, const QDateTime&) const
  *  \returns program location or -1 for not found.
  */
-int LiveTVChain::ProgramIsAt(const QString &chanid,
-                             const QDateTime &starttime) const
+int LiveTVChain::ProgramIsAt(uint chanid, const QDateTime &starttime) const
 {
     QMutexLocker lock(&m_lock);
 
     int count = 0;
-    QValueList<LiveTVChainEntry>::const_iterator it;
+    QList<LiveTVChainEntry>::const_iterator it;
     for (it = m_chain.begin(); it != m_chain.end(); ++it, ++count)
     {
         if ((*it).chanid == chanid &&
@@ -306,12 +332,12 @@ int LiveTVChain::ProgramIsAt(const QString &chanid,
     return -1;
 }
 
-/** \fn LiveTVChain::ProgramIsAt(const ProgramInfo*) const
+/** \fn LiveTVChain::ProgramIsAt(const ProgramInfo&) const
  *  \returns program location or -1 for not found.
  */
-int LiveTVChain::ProgramIsAt(const ProgramInfo *pginfo) const
+int LiveTVChain::ProgramIsAt(const ProgramInfo &pginfo) const
 {
-    return ProgramIsAt(pginfo->chanid, pginfo->recstartts);
+    return ProgramIsAt(pginfo.GetChanID(), pginfo.GetRecordingStartTime());
 }
 
 /** \fn LiveTVChain::GetLengthAtCurPos(void)
@@ -324,7 +350,7 @@ int LiveTVChain::GetLengthAtCurPos(void)
 
     entry = m_chain[m_curpos];
     if (m_curpos == ((int)m_chain.count() - 1))
-        return entry.starttime.secsTo(QDateTime::currentDateTime());
+        return entry.starttime.secsTo(MythDate::current());
     else
         return entry.starttime.secsTo(entry.endtime);
 }
@@ -334,15 +360,12 @@ int LiveTVChain::TotalSize(void) const
     return m_chain.count();
 }
 
-void LiveTVChain::SetProgram(ProgramInfo *pginfo)
+void LiveTVChain::SetProgram(const ProgramInfo &pginfo)
 {
-    if (!pginfo)
-        return;
-
     QMutexLocker lock(&m_lock);
 
-    m_cur_chanid = pginfo->chanid;
-    m_cur_startts = pginfo->recstartts;
+    m_cur_chanid  = pginfo.GetChanID();
+    m_cur_startts = pginfo.GetRecordingStartTime();
 
     m_curpos = ProgramIsAt(pginfo);
     m_switchid = -1;
@@ -371,13 +394,17 @@ void LiveTVChain::ClearSwitch(void)
  *
  *   NOTE: The caller is resposible for deleting the ProgramInfo
  */
-ProgramInfo *LiveTVChain::GetSwitchProgram(bool &discont, bool &newtype, 
+ProgramInfo *LiveTVChain::GetSwitchProgram(bool &discont, bool &newtype,
                                            int &newid)
 {
+    ReloadAll();
     QMutexLocker lock(&m_lock);
 
     if (m_switchid < 0 || m_curpos == m_switchid)
+    {
+        ClearSwitch();
         return NULL;
+    }
 
     LiveTVChainEntry oldentry, entry;
     GetEntryAt(m_curpos, oldentry);
@@ -386,7 +413,26 @@ ProgramInfo *LiveTVChain::GetSwitchProgram(bool &discont, bool &newtype,
     while (!pginfo && m_switchid < (int)m_chain.count() && m_switchid >= 0)
     {
         GetEntryAt(m_switchid, entry);
-        pginfo = EntryToProgram(entry);
+
+        bool at_last_entry = 
+            ((m_switchid > m_curpos) &&
+             (m_switchid == (int)(m_chain.count()-1))) ||
+            ((m_switchid <= m_curpos) && (m_switchid == 0));
+
+        // Skip dummy recordings, if possible.
+        if (at_last_entry || (entry.cardtype != "DUMMY"))
+            pginfo = EntryToProgram(entry);
+
+        // Skip empty recordings, if possible
+        if (pginfo && (0 == pginfo->GetFilesize()) &&
+            m_switchid < (int)(m_chain.count()-1))
+        {
+            LOG(VB_GENERAL, LOG_WARNING,
+                QString("Skipping empty program %1")
+                .arg(pginfo->MakeUniqueKey()));
+            delete pginfo;
+            pginfo = NULL;
+        }
 
         if (!pginfo)
         {
@@ -398,42 +444,24 @@ ProgramInfo *LiveTVChain::GetSwitchProgram(bool &discont, bool &newtype,
     }
 
     if (!pginfo)
-        return NULL;
-
-    // Skip dummy recordings, if possible.
-    if (entry.cardtype == "DUMMY")
     {
-        if (m_switchid > m_curpos && m_switchid + 1 < (int)m_chain.count())
-            m_switchid++;
-        else if (m_switchid < m_curpos && m_switchid > 0)
-            m_switchid--;
-
-        GetEntryAt(m_switchid, entry);
-        pginfo = EntryToProgram(entry);
-    }
-
-    if (!pginfo)
+        ClearSwitch();
         return NULL;
+    }
 
     discont = true;
     if (m_curpos == m_switchid - 1)
         discont = entry.discontinuity;
 
-    newtype = (oldentry.cardtype !=  entry.cardtype);
+    newtype = (oldentry.cardtype != entry.cardtype);
 
+    // Some cards can change their streams dramatically on a channel change...
     if (discont)
-    {
-        // Some cards can change their streams
-        // dramatically on a channel change...
-        newtype |= entry.cardtype == "DVB";
-        newtype |= entry.cardtype == "HDTV";
-        newtype |= entry.cardtype == "FIREWIRE";
-        newtype |= entry.cardtype == "DBOX2";
-        newtype |= entry.cardtype == "HDHOMERUN";
-    }
+        newtype |= CardUtil::IsChannelChangeDiscontinuous(entry.cardtype);
 
     newid = m_switchid;
-    m_switchid = -1;
+
+    ClearSwitch();
 
     return pginfo;
 }
@@ -446,7 +474,7 @@ void LiveTVChain::SwitchTo(int num)
 {
     QMutexLocker lock(&m_lock);
 
-    VERBOSE(VB_PLAYBACK, LOC + "SwitchTo("<<num<<")");
+    LOG(VB_PLAYBACK, LOG_INFO, LOC + QString("SwitchTo(%1)").arg(num));
 
     int size = m_chain.count();
     if ((num < 0) || (num >= size))
@@ -458,15 +486,17 @@ void LiveTVChain::SwitchTo(int num)
         GetEntryAt(num, m_switchentry);
     }
     else
-        VERBOSE(VB_IMPORTANT, LOC + "SwitchTo() not switching to current");   
+        LOG(VB_GENERAL, LOG_ERR, LOC + "SwitchTo() not switching to current");
 
-    if (print_verbose_messages & VB_PLAYBACK)
+    if (VERBOSE_LEVEL_CHECK(VB_PLAYBACK, LOG_DEBUG))
     {
         LiveTVChainEntry e;
         GetEntryAt(num, e);
         QString msg = QString("%1_%2")
-            .arg(e.chanid).arg(e.starttime.toString("yyyyMMddhhmmss"));
-        VERBOSE(VB_PLAYBACK, LOC + "Entry@"<<num<<": '"<<msg<<"'");
+            .arg(e.chanid)
+            .arg(MythDate::toString(e.starttime, MythDate::kFilename));
+        LOG(VB_PLAYBACK, LOG_DEBUG,
+            LOC + QString("Entry@%1: '%2')").arg(num).arg(msg));
     }
 }
 
@@ -477,7 +507,9 @@ void LiveTVChain::SwitchTo(int num)
  */
 void LiveTVChain::SwitchToNext(bool up)
 {
-    //VERBOSE(VB_PLAYBACK, LOC + "SwitchToNext("<<(up?"up":"down")<<")");
+#if 0
+    LOG(VB_PLAYBACK, LOG_DEBUG, LOC + "SwitchToNext("<<(up?"up":"down")<<")");
+#endif
     if (up && HasNext())
         SwitchTo(m_curpos + 1);
     else if (!up && HasPrev())
@@ -534,24 +566,45 @@ void LiveTVChain::SetHostSocket(MythSocket *sock)
 {
     QMutexLocker lock(&m_sockLock);
 
-    if (!m_inUseSocks.containsRef(sock))
+    if (!m_inUseSocks.contains(sock))
         m_inUseSocks.append(sock);
 }
 
-bool LiveTVChain::IsHostSocket(MythSocket *sock)
+bool LiveTVChain::IsHostSocket(const MythSocket *sock) const
 {
     QMutexLocker lock(&m_sockLock);
-    return m_inUseSocks.containsRef(sock);
+    return m_inUseSocks.contains(const_cast<MythSocket*>(sock));
 }
 
-int LiveTVChain::HostSocketCount(void)
+uint LiveTVChain::HostSocketCount(void) const
 {
+    QMutexLocker lock(&m_sockLock);
     return m_inUseSocks.count();
 }
 
 void LiveTVChain::DelHostSocket(MythSocket *sock)
 {
     QMutexLocker lock(&m_sockLock);
-    m_inUseSocks.removeRef(sock);
+    m_inUseSocks.removeAll(sock);
 }
 
+static QString toString(const LiveTVChainEntry &v)
+{
+    return QString("%1: %2 (%3 to %4)%5")
+        .arg(v.cardtype,6).arg(v.chanid,4)
+        .arg(v.starttime.time().toString())
+        .arg(v.endtime.time().toString())
+        .arg(v.discontinuity?" discontinuous":"");
+}
+
+QString LiveTVChain::toString() const
+{
+    QMutexLocker lock(&m_lock);
+    QString ret = QString("LiveTVChain has %1 entries\n").arg(m_chain.size());
+    for (uint i = 0; i < (uint)m_chain.size(); i++)
+    {
+        ret += (QString((i==(uint)m_curpos) ? "* " : "  ") +
+                ::toString(m_chain[i]) + "\n");
+    }
+    return ret;
+}
