@@ -1,15 +1,20 @@
+#include <signal.h> // for signal
+
 #ifndef _WIN32
 #include <QCoreApplication>
 #else
 #include <QApplication>
 #endif
 
+#include <signal.h>
+
 #include <QFileInfo>
 #include <QRegExp>
 #include <QFile>
 #include <QDir>
 #include <QMap>
 
+#include "signalhandling.h"
 #include "commandlineparser.h"
 #include "scheduledrecording.h"
 #include "previewgenerator.h"
@@ -17,6 +22,7 @@
 #include "mythsystemevent.h"
 #include "backendcontext.h"
 #include "main_helpers.h"
+#include "mythmiscutil.h"
 #include "storagegroup.h"
 #include "housekeeper.h"
 #include "mediaserver.h"
@@ -46,12 +52,6 @@
     #define UNUSED_FILENO 3
 #endif
 
-static void qt_exit(int)
-{
-    signal(SIGINT, SIG_DFL);
-    QCoreApplication::exit(0);
-}
-
 int main(int argc, char **argv)
 {
     MythBackendCommandLineParser cmdline;
@@ -99,8 +99,14 @@ int main(int argc, char **argv)
         close(0);
 
     CleanupGuard callCleanup(cleanup);
-    signal(SIGINT, qt_exit);
-    signal(SIGTERM, qt_exit);
+
+#ifndef _WIN32
+    QList<int> signallist;
+    signallist << SIGINT << SIGTERM << SIGSEGV << SIGABRT << SIGBUS << SIGFPE
+               << SIGILL << SIGRTMIN;
+    SignalHandler::Init(signallist);
+    signal(SIGHUP, SIG_IGN);
+#endif
 
     gContext = new MythContext(MYTH_BINARY_VERSION);
     if (!gContext->Init(false))
