@@ -7,6 +7,7 @@ using namespace std;
 #include <QCoreApplication>
 #include <QFile>
 
+#include "mythdate.h"
 #include "exitcodes.h"
 #include "mythcontext.h"
 #include "mythdb.h"
@@ -20,6 +21,7 @@ using namespace std;
 #include "mythlogging.h"
 #include "commandlineparser.h"
 #include "programinfo.h"
+#include "signalhandling.h"
 
 static void setGlobalSetting(const QString &key, const QString &value)
 {
@@ -203,7 +205,9 @@ static QDateTime getDailyWakeupTime(QString sPeriod)
 {
     QString sTime = getGlobalSetting(sPeriod, "00:00");
     QTime tTime = QTime::fromString(sTime, "hh:mm");
-    QDateTime dtDateTime = QDateTime(QDate::currentDate(), tTime);
+    QDateTime dtDateTime = QDateTime(
+        MythDate::current().toLocalTime().date(),
+        tTime, Qt::LocalTime).toUTC();
 
     return dtDateTime;
 }
@@ -271,7 +275,7 @@ static int getStatus(bool bWantRecStatus)
     QDateTime dtPeriod1End = getDailyWakeupTime("DailyWakeupEndPeriod1");
     QDateTime dtPeriod2Start = getDailyWakeupTime("DailyWakeupStartPeriod2");
     QDateTime dtPeriod2End = getDailyWakeupTime("DailyWakeupEndPeriod2");
-    QDateTime dtCurrent = QDateTime::currentDateTime();
+    QDateTime dtCurrent = MythDate::current();
 
     // Check for time periods that cross midnight
     if (dtPeriod1End < dtPeriod1Start)
@@ -380,7 +384,7 @@ static int setWakeupTime(QString sWakeupTime)
 
     // check time given is valid
     QDateTime dtWakeupTime;
-    dtWakeupTime = QDateTime::fromString(sWakeupTime, Qt::ISODate);
+    dtWakeupTime = MythDate::fromString(sWakeupTime);
 
     if (!dtWakeupTime.isValid())
     {
@@ -393,7 +397,7 @@ static int setWakeupTime(QString sWakeupTime)
     }
 
     setGlobalSetting("MythShutdownNextScheduled",
-                     dtWakeupTime.toString(Qt::ISODate));
+                     MythDate::toString(dtWakeupTime, MythDate::kDatabase));
 
     return 0;
 }
@@ -444,7 +448,7 @@ static int shutdown()
     QDateTime dtPeriod1End = getDailyWakeupTime("DailyWakeupEndPeriod1");
     QDateTime dtPeriod2Start = getDailyWakeupTime("DailyWakeupStartPeriod2");
     QDateTime dtPeriod2End = getDailyWakeupTime("DailyWakeupEndPeriod2");
-    QDateTime dtCurrent = QDateTime::currentDateTime();
+    QDateTime dtCurrent = MythDate::current();
     QDateTime dtNextDailyWakeup = QDateTime();
 
     // Make sure Period1 is before Period2
@@ -481,7 +485,8 @@ static int shutdown()
         if (dtCurrent < dtPeriod1Start)
         {
             LOG(VB_GENERAL, LOG_NOTICE,
-                "daily wakeup today at " + dtPeriod1Start.toString("hh:mm:ss"));
+                "daily wakeup today at " +
+                dtPeriod1Start.toLocalTime().toString("hh:mm:ss"));
             dtNextDailyWakeup = dtPeriod1Start;
         }
     }
@@ -492,7 +497,8 @@ static int shutdown()
         if (dtCurrent < dtPeriod2Start)
         {
             LOG(VB_GENERAL, LOG_NOTICE,
-                "daily wakeup today at " + dtPeriod2Start.toString("hh:mm:ss"));
+                "daily wakeup today at " +
+                dtPeriod2Start.toLocalTime().toString("hh:mm:ss"));
             dtNextDailyWakeup = dtPeriod2Start;
         }
     }
@@ -512,7 +518,7 @@ static int shutdown()
             dtNextDailyWakeup = dtNextDailyWakeup.addDays(1);
 
             LOG(VB_GENERAL, LOG_NOTICE, "next daily wakeup is tomorrow at " +
-                dtNextDailyWakeup.toString("hh:mm:ss"));
+                dtNextDailyWakeup.toLocalTime().toString("hh:mm:ss"));
         }
     }
 
@@ -524,7 +530,7 @@ static int shutdown()
     QDateTime dtNextRecordingStart = QDateTime();
     QString s = getGlobalSetting("MythShutdownNextScheduled", "");
     if (!s.isEmpty())
-        dtNextRecordingStart = QDateTime::fromString(s, Qt::ISODate);
+        dtNextRecordingStart = MythDate::fromString(s);
 
     if (!dtNextRecordingStart.isValid())
         LOG(VB_GENERAL, LOG_ERR, "no recording time is set");
@@ -598,7 +604,7 @@ static int shutdown()
 
     // save the next wakuptime in the db
     setGlobalSetting("MythShutdownWakeupTime",
-                     dtWakeupTime.toString(Qt::ISODate));
+                     MythDate::toString(dtWakeupTime, MythDate::kDatabase));
 
     // stop here to debug
     //return 0;
@@ -627,7 +633,8 @@ static int shutdown()
             }
             else
                 nvramCommand.replace(
-                    "$time", dtWakeupTime.toString(wakeup_timeformat));
+                    "$time", dtWakeupTime.toLocalTime()
+                    .toString(wakeup_timeformat));
 
             LOG(VB_GENERAL, LOG_NOTICE, "sending command to set time in bios" +
                 nvramCommand);
@@ -714,7 +721,7 @@ static int startup()
     QDateTime startupTime = QDateTime();
     QString s = getGlobalSetting("MythshutdownWakeupTime", "");
     if (!s.isEmpty())
-        startupTime = QDateTime::fromString(s, Qt::ISODate);
+        startupTime = MythDate::fromString(s);
 
     // if we don't have a valid startup time assume we were started manually
     if (!startupTime.isValid())
@@ -724,7 +731,7 @@ static int startup()
         // if we started within 15mins of the saved wakeup time assume we started
         // automatically to record or for a daily wakeup/shutdown period
 
-        int delta = startupTime.secsTo(QDateTime::currentDateTime());
+        int delta = startupTime.secsTo(MythDate::current());
         if (delta < 0)
             delta = -delta;
 
@@ -777,11 +784,20 @@ int main(int argc, char **argv)
     if ((retval = cmdline.ConfigureLogging(mask)) != GENERIC_EXIT_OK)
         return retval;
 
+#ifndef _WIN32
+    QList<int> signallist;
+    signallist << SIGINT << SIGTERM << SIGSEGV << SIGABRT << SIGBUS << SIGFPE
+               << SIGILL << SIGRTMIN;
+    SignalHandler::Init(signallist);
+    signal(SIGHUP, SIG_IGN);
+#endif
+
     gContext = new MythContext(MYTH_BINARY_VERSION);
     if (!gContext->Init(false))
     {
         LOG(VB_GENERAL, LOG_ERR,
             "mythshutdown: Could not initialize MythContext. Exiting.");
+        SignalHandler::Done();
         return GENERIC_EXIT_NO_MYTHCONTEXT;
     }
 
@@ -819,6 +835,8 @@ int main(int argc, char **argv)
 
     delete gContext;
 
+    SignalHandler::Done();
+
     return res;
 }
 
